/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2009  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c (for Renesas Starterkit RSKH8S2215R and HEW compiler)
Purpose : BSP for RSKH8S2215R
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/

/****** SFRs and identifiers used for LED demo **********************/
#define _PFDDR        (*(volatile unsigned char *) (0xFFFE3E)) /* Port F data direction register */
#define _LEDPORT	(*(volatile unsigned char *) (0xFFFF0E)) /* Port F data register           */

/****** Assign LEDs to Ports ****************************************/
#define _LED0_BIT      (1) /* LED1  */
#define _LED1_BIT      (2) /* LED2  */
#define _LEDMASK_ALL   ((1 << _LED0_BIT) | (1 << _LED1_BIT))

/****** Target specific configuration *******************************/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       LED_Init()
*/
void BSP_Init(void) {
  _PFDDR    |=  ((1 << _LED0_BIT) | (1 << _LED1_BIT));    // Enable LED output ports
  _LEDPORT  &= ~((1 << _LED0_BIT) | (1 << _LED1_BIT));    // Switch off both LEDs
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  _LEDPORT |= (1 << (Index+1));         /* Switch on LED 0  */
}

void BSP_ClrLED(int Index) {
  _LEDPORT &= ~(1 << (Index+1));        /* Switch off LED 0 */
}

void BSP_ToggleLED(int Index) {
  _LEDPORT ^= (1 << (Index+1));         /* Switch off LED 0 */
}                          

/****** EOF *********************************************************/


