/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2009  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c (for Renesas Starterkit RSKH8S2456 and HEW compiler)
Purpose : BSP for RSKH8S2456
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"
#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/

/****** SFRs and identifiers used for LED demo **********************/
#define P6DDR       (*(volatile OS_U8 *) (0xFFFE25)) /* Port 6 data direction register */
#define LEDPORT     (*(volatile OS_U8 *) (0xFFFF65)) /* Port 6 data register           */

/****** Assign LEDs to Ports ****************************************/
#define	LED0					P6.DR.BIT.B3
#define	LED1					P6.DR.BIT.B5
#define	LED2					PJ.DR.BIT.B0
#define	LED3					PJ.DR.BIT.B1

#define LED0_BIT      (3) /* LED0  */
#define LED1_BIT      (5) /* LED1  */
#define LEDMASK_ALL   ((1 << LED0_BIT) | (1 << LED1_BIT))

/****** Target specific configuration *******************************/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       LED_Init()
*/
void BSP_Init(void) {
  P6DDR    |=  LEDMASK_ALL;    // Enable LED output ports
  LEDPORT  |=  LEDMASK_ALL;    // Switch off all LEDs
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  OS_IncDI();
  if (Index == 0) {
    LEDPORT &= ~(1 << (LED0_BIT));        /* Switch on LED1 */  
  } else if (Index == 1) {
    LEDPORT &= ~(1 << (LED1_BIT));        /* Switch on LED2 */  
  }
  OS_DecRI(); 
}

void BSP_ClrLED(int Index) {
  OS_IncDI();
  if (Index == 0) {
    LEDPORT &= ~(1 << (LED0_BIT));        /* Switch off LED1 */  
  } else if (Index == 1) {
    LEDPORT &= ~(1 << (LED1_BIT));        /* Switch off LED2 */  
  }
  OS_DecRI(); 
}

void BSP_ToggleLED(int Index) {
  OS_IncDI();
  if (Index == 0) {
    LEDPORT ^= (1 << (LED0_BIT));         /* Toogle LED1 */  
  } else if (Index == 1) {
    LEDPORT ^= (1 << (LED1_BIT));         /* Toogle LED2 */  
  }
  OS_DecRI(); 
}

/****** EOF *********************************************************/


