/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2009  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_H8S2456.c   (for Renesas H8S2456 and HEW compiler)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.H"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/


#ifndef   OS_FSYS                   /* CPU main clock frequncy 16 Mhz x 2 (PLL) = 32 MHz */
  #define OS_FSYS 32000000
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */ 
  #define OS_PCLK_TIMER (OS_FSYS)   /* Normally derived from CPU    */
#endif                              /* clock                        */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART  OS_FSYS     /* Normally derived from CPU    */
#endif                              /* clock                        */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ  (1000)
#endif
  
/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART0 per default
*       and supports UART0 (SCIF)
*/
#ifndef   OS_UART
  #define OS_UART 1     
#endif    
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

#define BRR_VAL ((OS_PCLK_UART+OS_BAUDRATE*16L)/(OS_BAUDRATE*32L)-1)
#if BRR_VAL > 0xFF
  #error "UART can not be used with standard intialization."  
#endif

#define OS_UART_USED (OS_UART == 1)

#define OS_TIMER_RELOAD (OS_PCLK_TIMER/OS_TICK_FREQ)
#if (OS_TIMER_RELOAD >= 0x10000)
  #error "FRT Timer can not be used, please check configuration"
#endif

/*********************************************************************
*
*       SFR definition of sfrs used in RTOSInit.c
*
*       We define sfrs here to be independend of any i/o header file
*/
#define SCKCR      (*(volatile OS_U8  *)(0xFFFF3B))  /* System clock control register   */
#define PLLCR      (*(volatile OS_U8  *)(0xFFFF45))  /* PLL Control register            */
#define USPLLCR    (*(volatile OS_U8  *)(0xFFFF45))  /* USB PLL Control register        */
#define INTCR      (*(volatile OS_U8  *)(0xFFFF31))  /* Interrupt control register      */ 
#define MSTPCRH    (*(volatile OS_U8  *)(0xFFFF40))  /* Modul standby register high     */
#define MSTPCRL    (*(volatile OS_U8  *)(0xFFFF41))  /* Modul standby register low      */
#define IPRF       (*(volatile OS_U16 *)(0xFFFE0A))  /* Interrupt priority register F   */
#define IPRJ       (*(volatile OS_U16 *)(0xFFFE12))  /* Interrupt priority register J   */

#define TPU_TSTR   (*(volatile OS_U8  *)(0xFFFFC0))  /* Timer start register            */
#define TPU0_TCR   (*(volatile OS_U8  *)(0xFFFFD0))  /* Timer control register_0        */
#define TPU0_TMDR  (*(volatile OS_U8  *)(0xFFFFD1))  /* Timer Mode register             */
#define TPU0_TIOR  (*(volatile OS_U8  *)(0xFFFFD2))  /* Timer I/O control register      */
#define TPU0_TIER  (*(volatile OS_U8  *)(0xFFFFD4))  /* Timer interrupt enable register */
#define TPU0_TSR   (*(volatile OS_U8  *)(0xFFFFD5))  /* Timer status register           */
#define TPU0_CNT   (*(volatile OS_U16 *)(0xFFFFD6))  /* Timer count register            */
#define TPU0_TGRA  (*(volatile OS_U16 *)(0xFFFFD8))  /* Timer general register A        */
#define TPU0_TGRB  (*(volatile OS_U16 *)(0xFFFFDA))  /* Timer general register B        */

#define SMR_1      (*(volatile OS_U8 *)(0xFFFF80))   /* Serial mode register            */
#define BRR_1      (*(volatile OS_U8 *)(0xFFFF81))   /* Bit rate register               */ 
#define SCR_1      (*(volatile OS_U8 *)(0xFFFF82))   /* Serial control register         */
#define TDR_1      (*(volatile OS_U8 *)(0xFFFF83))   /* Transmit data register          */
#define SSR_1      (*(volatile OS_U8 *)(0xFFFF84))   /* Serial status register          */
#define RDR_1      (*(volatile OS_U8 *)(0xFFFF85))   /* Receive data register           */

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/
/*********************************************************************
*
*       OS_ISR_Tick_Handler()
*
*       embOS timer interrupt handler
*/
void OS_ISR_Tick_Handler(void) {
  TPU0_TSR &= ~(1 << 0);              // Clear the interrupt flag 
  OS_HandleTick();
}

/*********************************************************************
*
*       OS_ISR_Tick()
*
*       embOS timer interrupt handler
*/
__interrupt(vect=40) void OS_ISR_Tick (void) {  
  OS_CallISR(&OS_ISR_Tick_Handler);
}  

/*********************************************************************
*
*       OS_InitHW
*
*
*  Initialize hardware required for embOS to run.
*  1. We switch CPU into interrupt mode 1 
*  2. We initialize a timer for embOS with low interrupt priority.
*     We use 16bit FRT Counter in output compare mode.
*     This function should work on any hardware, but may have to be modified,
*     if you decide to use an other timer.   
*/
void OS_InitHW(void) {
  volatile OS_U32 DelayCnt;
  OS_IncDI();                         // Initially disable interrupts
	/* 
  *   Frequency Multiplication Factor Switching Mode Select.
	*   Specified multiplication factor is valid immediately after STC1 and 
	*   STC0 bits are rewritten. 
  */
	SCKCR |= (1 << 3);
	/*  
  *   Frequency Multiplication Factor for System Clock PLL circuit
	*   Multiplication factor = '2' selected.
	*   System-clock PLL circuit = 16MHz
	*   Multiplication factor = 2 
	*   System clock to 'PHI' pin = System-clock PLL circuit (16MHz) * 2 = 32MHz 
  */	
	PLLCR |= (1 << 0);
	for (DelayCnt=0; DelayCnt < 0x100000; DelayCnt++); // Add delay for PLL to stabilise. 
	/* 
  *   Frequency Multiplication Factor for USB PLL Circuit Setting
	*   Multiplication factor = '3' selected.
	*   System-clock PLL circuit = 16MHz
	*   Multiplication factor = 3
	*   USB dedicated clock = System-clock PLL circuit (16MHz) * 3 = 48MHz 
	*/
	USPLLCR |= 0x01;
	for (DelayCnt=0; DelayCnt < 0x100000; DelayCnt++); // Add delay for PLL to stabilise. 
  INTCR |= (1uL << 5);                // Set Interrupt mode 2 
  MSTPCRH &= ~(1uL << 3);             // Disable standby for Timer TPU0
  TPU_TSTR = 0x00;                    // Stop the timer 
  TPU0_TCR = 0x20;                    // TPSC2:TPSC1:TPSC0 - 011 (Internal clock: counts on PHI/64) 
                                      // CKEG1:CKEG0 - 00 (Count at rising edge) 
                                      // CCLR2:CCLR1:CCLR0 - 001 (TCNT cleared by TGRA compare match/input capture) 
  TPU0_TMDR = 0x00;                   // MD3:MD2:MD1:MD0	- 0000 (Normal operation)
                                      // BFA - 0 (TGRA operates normally)
                                      // BFB - 0 (TGRB operates normally)
  TPU0_TIER = 0x43;                   // Timer interrupt enable register
  TPU0_TGRA = OS_TIMER_RELOAD;        // Initialize the Timer General Register 
  IPRF     &= ~(7uL << 4);            // Clear interrupt priority bits
  IPRF     |= (1uL << 4);             // Set interrupt priority 
  TPU_TSTR |= 1;                      // Start the timer 
  OS_COM_Init();
  OS_DecRI();                         // Restore low priority interrupt enable state
}

/*********************************************************************
*
*       OS_GetTimeCcycles()
*
*  This routine is required for task-info via embOSView
*  or high precision time measurment functions.
*  It returns the system time in clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U16 Cnt;
  OS_U32 Time;
  Cnt = TPU0_CNT;
  Time = OS_Time;
  if (TPU0_TSR & (1 << 0)) { // Re-read if interrupt pending
    Time++;
    Cnt = TPU0_CNT;
  }
  return (OS_PCLK_TIMER/1000) * Time + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*  
*  Purpose:
*     Convert Cycles into micro seconds.
*  
*     If your clock frequency is not a multiple of 1 MHz, you may have to
*     modify this routine in order to get proper diagonstics.
*  Please note:
*     This routine is required for profiling or high resolution time
*     measurement. It does not affect operation of embOS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/((OS_PCLK_TIMER)/1000000);
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*  
*  Please note:
*  This is basically the "core" of the idle loop
*  This core loop can be changed, but:
*  The idle loop does not have a stack of its own, therefore no
*  functionality should be implemented that relies on the stack
*  to be preserved. However, a simple program loop can be programmed
*  (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) { // Idle loop: No task is ready to exec
  for (;;);          // Nothing to do ... wait for an interrupt
}

#if (OS_UART_USED)

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for communication with embOSView
*/
void OS_COM_Init(void) {
  MSTPCRL &=  ~(1 << 2);    // Enable SIO module (clear stop mode) 
  SMR_1    = 0x00;          // Select 8N1
  BRR_1    = BRR_VAL;       // Select baudrate
  IPRJ    &= ~(7uL << 12);  // Clear interrupt priority bits
  IPRJ    |= (1uL << 12);   // Set interrupt priority
  SCR_1    = 0x70;          // Rx interrupt and Rx, Tx enable
}

/*********************************************************************
*
*       OS_COM_Send1() for UART0: Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  TDR_1  = c;
  SCR_1 |= 0x80;   // Tx interrupt enable
  SSR_1 &= ~0x80;  // Reset pending flag  
}

/*********************************************************************
*
*       OS_UartError_Handler
*
*/
void OS_UartError_Handler(void) {
  SSR_1 &= ~((1<<5) | (1<<4) | (1<<3));
}

/*********************************************************************
*
*       OS_Uart_Error_ISR() 
*      
*       Uart Error ISR
*/
__interrupt(vect=92) void OS_UartError_ISR (void) {
  OS_CallISR(&OS_UartError_Handler);
}

/*********************************************************************
*
*       OS_UartRx_Handler
*
*/
void OS_UartRx_Handler(void) {
  SSR_1 &= ~0x40;  // Reset pending flag
  OS_OnRx(RDR_1);  
}

/*********************************************************************
*
*       OS_Uart_Rx_ISR() 
*      
*       Uart Rx ISR
*/
__interrupt(vect=93) void OS_UartRx_ISR (void) {
  OS_CallISR(&OS_UartRx_Handler);
}

/*********************************************************************
*
*       OS_UartTx_Handler
*
*/
void OS_UartTx_Handler(void) {
  SSR_1 &= ~0x80;    // Reset pending flag
  if (OS_OnTx()) {   // No more characters to send ?
    SCR_1 &= ~0x80;  // Disable Tx Interrupt
  }
}

/*********************************************************************
*
*       OS_Uart_Tx_ISR() 
*      
*       Uart Tx ISR
*/
__interrupt(vect=94) void OS_UartTx_ISR (void) {
  OS_CallISR(&OS_UartTx_Handler);
}

/*********************************************************************
*
*       Communication for embOSView not selected
*
*/
#else  /* No communication routines required, use dummies  */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
  }

#endif /* OS_UART_USED */

/*****  EOF  ********************************************************/
