/******************************************************************************
* DISCLAIMER:
* The software supplied by Renesas Technology Europe Ltd is
* intended and supplied for use on Renesas Technology products.
* This software is owned by Renesas Technology Europe, Ltd. Or
* Renesas Technology Corporation and is protected under applicable
* copyright laws. All rights are reserved.
*
* THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS,
* IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* APPLY TO THIS SOFTWARE. RENESAS TECHNOLOGY AMERICA, INC. AND
* AND RENESAS TECHNOLOGY CORPORATION RESERVE THE RIGHT, WITHOUT
* NOTICE, TO MAKE CHANGES TO THIS SOFTWARE. NEITHER RENESAS
* TECHNOLOGY AMERICA, INC. NOR RENESAS TECHNOLOGY CORPORATION SHALL,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
* CONSEQUENTIAL DAMAGES FOR ANY REASON WHATSOEVER ARISING OUT OF THE
* USE OR APPLICATION OF THIS SOFTWARE.
******************************************************************************/

/* Copyright (C) 2009. Renesas Technology Europe, All Rights Reserved */

/*FILE COMMENT******************************* Technical reference data ****
* File Name		: resetprg.c
* Version 		: 1.00
* Device 		: H8S/2456R (R4F24569VFQV)
* Tool Chain 	: HEW, Hitachi H8S, H8/300 Standard Toolchain v6.2
* H/W Platform	: RSK2H8S2456R
* Description 	: Reset program
******************************************************************************/

/******************************************************************************
* History 		: 17.06.2009 Ver. 1.00 First Release
*FILE COMMENT END*********************************************************/

/******************************************************************************
System Includes
******************************************************************************/
/* 	This header file provides access to intrinsic functions useful for configuring
	and controlling the interrupts. */
#include <machine.h>
/* Following header file provides prototypes for interrupt configuring 
   functions */
#include <_h_c_lib.h>

/******************************************************************************
User Includes (Project level includes)
******************************************************************************/
/* Following header file is used by __entry directive to set the initial 
   stack pointer. The value in this file should only be changed if relocating
   and / or changing the size of the stack. */
#include "stacksct.h"

/***********************************************************************************
Function Prototypes
***********************************************************************************/
extern void main(void);
extern void HardwareSetup(void);

void PowerON_Reset(void);

/******************************************************************************
User Program Code
******************************************************************************/

/******************************************************************************
Compiler Directives
******************************************************************************/
/* '#pragma section' instructs the linker to place all subsequent code in the 
named section. This can be used to relocate code in the memory map. 
Use of the directive without the section name will reset the section to 
default. */

/*FUNC COMMENT*************************************************************
* Outline 		: PowerON_Reset
* Description 	: Power on reset function. This function executes following to 
*				  power on reset. It first calls hardware initialisation 
*				  function & then 'main()' function. 
* Argument  	: none
* Return value  : none
*FUNC COMMENT END*********************************************************/

#pragma section ResetPRG

/* Note: The keyword '__entry' specifies that, the function following this 
  keyword is an entry function of the application. This will be the first function
  to be executed after CPU reset.   */
  
__entry(vect=0)
void PowerON_Reset(void)
{ 
	/* Renesas H8/SH Compiler Intrinsic function to copy initialised data to RAM */
	/* This function references the structure in the 'dbsct.c' source file */
	_INITSCT();
	
	/* Mask interrupts */
	set_imask_ccr((unsigned char)1);

	/*	Setup the hardware.	*/
	//HardwareSetup();				

	/*	Call user 'main()'.	*/
	main();

	/*	This function must never exit.	*/
	while(1);
}
/***********************************************************************************
End of function PowerON_Reset
***********************************************************************************/
