/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2009  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c (for Renesas Starterkit RSKH8SX1664F and HEW compiler)
Purpose : BSP for RSKH8SX1664F
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"
#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/


/****** SFRs and identifiers used for LED demo **********************/


/****** Assign LEDs to Ports ****************************************/

/****** Target specific configuration *******************************/

#define _PCDDR         (*(volatile unsigned char *) (0xFFFB8B)) /* Port C data direction register */
#define _LEDPORT       (*(volatile unsigned char *) (0xFFFF5B)) /* Port C data register           */
#define _LED0_BIT      (2) /* LED1  */
#define _LED1_BIT      (3) /* LED2  */
#define _LEDMASK_ALL   ((1 << _LED0_BIT) | (1 << _LED1_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       LED_Init()
*/

void BSP_Init(void) {
  _PCDDR    |=  ((1 << _LED0_BIT) | (1 << _LED1_BIT));    // Enable LED output ports
  _LEDPORT  |= ((1 << _LED0_BIT) | (1 << _LED1_BIT));     // Switch off both LEDs
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  OS_IncDI();
  _LEDPORT &= ~(1 << (Index+2));        /* Switch on LED  */
  OS_DecRI();  
}

void BSP_ClrLED(int Index) {
  OS_IncDI();
  _LEDPORT |= (1 << (Index+2));         /* Switch off LED */
  OS_DecRI();  
}

void BSP_ToggleLED(int Index) {
  OS_IncDI();  
  _LEDPORT ^= (1 << (Index+2));         /* Toogle LED     */
  OS_DecRI();  
}

/****** EOF *********************************************************/


